@@ -1,5 +1,5 @@
 import logging
-from typing import List
+import math
 from pvpower.estimator import Estimator, SVMEstimator, FullVectorizer, CoreVectorizer, SunshineVectorizer, PlusVisibilityVectorizer, SushinePlusCloudCoverVectorizer, PlusSunshineVectorizer, PlusCloudCoverVectorizer, PlusVisibilitySunshineVectorizer, PlusVisibilityCloudCoverVectorizer, PlusVisibilityFogCloudCoverVectorizer
 from pvpower.traindata import TrainData
 
@@ -14,37 +14,27 @@ class TrainRun:
         self.predictions = [estimator.predict(validation_record) for validation_record in self.validation_samples]
         self.estimator.retrain(train_data)  # retrain with all data
 
-    def __derivation(self, real, predicted):
-        if abs(real - predicted) < 10: # ignore diff < 10
-            return 0
-        elif real == 0:
-            return predicted
-        elif predicted == 0:
-            return real
-        else:
-            return real-predicted
-
+    def __score(self, real, predicted):
+        rounded_real = round(real/10)*10
+        rounded_predicted = round(predicted/10)*10
+        score = abs(rounded_real - rounded_predicted)
+        return score
 
     @property
     def score(self) -> float:
-        derivation_total = []
+        scores = []
         for i in range(len(self.validation_samples)):
-            if self.validation_samples[i].irradiance == 0:
-                continue
-            else:
-                predicted = self.predictions[i]
-            diff = self.__derivation(self.validation_samples[i].power_watt, predicted)
-            derivation_total.append(diff)
+            scores.append(self.__score(self.validation_samples[i].power_watt, self.predictions[i]))
 
-        derivations = sorted([abs(value) for value in derivation_total])
-        ignore_size = int(len(derivations) * 0.05)
+        scores = sorted(scores)
+        ignore_size = int(len(scores) * 0.1)
         if ignore_size <= 0:
             ignore_size = 1
-        derivations = derivations[ignore_size:-ignore_size]
-        if len(derivations) == 0:
+        scores = scores[ignore_size:-ignore_size]
+        if len(scores) == 0:
             return 999999
         else:
-            return round(sum(derivations) / len(derivations), 2)
+            return round(sum(scores) / len(scores), 2)
 
     def __lt__(self, other):
         return self.score < other.score
@@ -58,21 +48,21 @@ class TrainRun:
     def __str__(self):
         max_lines = 1000
         txt = str(self.estimator) + "\n"
-        txt += "derivation:  " + str(self.score) + " \n"
-        txt += '{:14s}        {:14s} {:14s} {:14s} {:14s} {:14s}         {:10s} {:10s}           {:10s}\n'.format("time", "irradiance", "sunshine", "cloud_cover", "visibility", "proba.fog", "real", "predicted", "derivation")
+        txt += "score:  " + str(self.score) + " (smaller is better)\n"
+        txt += '{:25s}   {:10s} {:10s}    {:10s}          {:14s} {:14s} {:14s} {:14s} {:14s}         \n'.format("time", "real", "predicted", "score", "irradiance", "sunshine", "cloud_cover", "visibility", "proba.fog")
         for i in range(0, len(self.validation_samples)):
             if i > max_lines:
                 txt += '....'
                 break
-            txt += '{:<14s}        {:<14d} {:<14d} {:<14d} {:<14d}  {:<14d}        {:<10d} {:<10d}           {:<10d}\n'.format(self.validation_samples[i].time.strftime("%d.%b  %H:%M"),
-                                                                                                                               self.validation_samples[i].irradiance,
-                                                                                                                               self.validation_samples[i].sunshine,
-                                                                                                                               self.validation_samples[i].cloud_cover_effective,
-                                                                                                                               self.validation_samples[i].visibility,
-                                                                                                                               self.validation_samples[i].probability_for_fog,
+            txt += '{:<25s}   {:<10d} {:<10d}    {:<10s}          {:<14d} {:<14d} {:<14s} {:<14d}  {:<14d}        \n'.format(self.validation_samples[i].time.strftime("%d.%b %H:%M") + " (" + self.validation_samples[i].time_utc.strftime("%H:%M") + " utc)",
                                                                                                                                self.validation_samples[i].power_watt,
                                                                                                                                self.predictions[i],
-                                                                                                                               int(self.__derivation(self.validation_samples[i].power_watt, self.predictions[i])))
+                                                                                                                               "[" + str(self.__score(self.validation_samples[i].power_watt, self.predictions[i])) + "]",
+                                                                                                                               self.validation_samples[i].irradiance,
+                                                                                                                               self.validation_samples[i].sunshine,
+                                                                                                                               str(self.validation_samples[i].cloud_cover_effective) + "%",
+                                                                                                                               self.validation_samples[i].visibility,
+                                                                                                                               self.validation_samples[i].probability_for_fog)
         return txt
 
 
@@ -91,6 +81,7 @@ class TrainingCenter:
                       SVMEstimator(PlusVisibilityCloudCoverVectorizer()),
                       SVMEstimator(PlusVisibilityFogCloudCoverVectorizer())]
 
+        logging.info("train estimators with " + str(len(trainData.samples)) + " samples")
         rounds = 6
         train_runs = []
         for estimator in estimators:
@@ -100,9 +91,9 @@ class TrainingCenter:
                 runs = cleaned_runs
             train_runs.append(runs[int(len(runs)*0.5)])
 
-        train_runs = sorted(train_runs)
+        train_runs = sorted(train_runs, reverse=True)
         for run in train_runs:
             logging.debug(run)
-        best_train = train_runs[0]
+        best_train = train_runs[-1]
         logging.info("new estimator trained \n" + str(best_train))
         return best_train.estimator
